@@ -1,7 +1,7 @@
 #include "moar.h"
 #include "platform/time.h"
 #include "platform/fork.h"
-#include "tinymt64.h"
+#include "core/jfs64.h"
 #include "bithacks.h"
 
 /* concatenating with "" ensures that only literal strings are accepted as argument. */
@@ -440,7 +440,7 @@ static void deferred_close_perform(MVMThreadContext *tc, uv_loop_t *loop, MVMObj
     }
 }
 
-MVMObject * get_async_task_handle(MVMThreadContext *tc, MVMOSHandle *h) {
+static MVMObject * get_async_task_handle(MVMThreadContext *tc, MVMOSHandle *h) {
     MVMIOAsyncProcessData *handle_data = (MVMIOAsyncProcessData *)h->body.data;
     return handle_data->async_task;
 }
@@ -1094,22 +1094,21 @@ MVMint64 MVM_proc_getppid(MVMThreadContext *tc) {
 
 /* generates a random int64 */
 MVMint64 MVM_proc_rand_i(MVMThreadContext *tc) {
-    MVMuint64 result = tinymt64_generate_uint64(tc->rand_state);
-    return *(MVMint64 *)&result;
+    return (MVMint64)jfs64_generate_uint64(tc->rand_state);
 }
 
 /* generates a number between 0 and 1 */
 MVMnum64 MVM_proc_rand_n(MVMThreadContext *tc) {
-    return tinymt64_generate_double(tc->rand_state);
+    return ((jfs64_generate_uint64(tc->rand_state) >> 11) * (1.0 / 9007199254740992.0));
 }
 
 MVMnum64 MVM_proc_randscale_n(MVMThreadContext *tc, MVMnum64 scale) {
-    return tinymt64_generate_double(tc->rand_state) * scale;
+    return ((jfs64_generate_uint64(tc->rand_state) >> 11) * (1.0 / 9007199254740992.0)) * scale;
 }
 
 /* seed random number generator */
 void MVM_proc_seed(MVMThreadContext *tc, MVMint64 seed) {
-    tinymt64_init(tc->rand_state, (MVMuint64)seed);
+    jfs64_init(tc->rand_state, (MVMuint64)seed);
 }
 
 /* gets the system time since the epoch in nanoseconds */
@@ -1251,9 +1250,9 @@ MVMint64 MVM_proc_fork(MVMThreadContext *tc) {
     /* Allow MVM_io_eventloop_start to restart the thread if necessary */
     instance->event_loop_thread = NULL;
 
-    MVM_gc_mark_thread_blocked(tc);
+    /* Do not mark thread blocked as the GC also tries to acquire
+     * mutex_threads and it's held only briefly by all holders anyway */
     uv_mutex_lock(&instance->mutex_threads);
-    MVM_gc_mark_thread_unblocked(tc);
 
     /* Check if we are single threaded and if true, fork() */
     if (MVM_thread_cleanup_threads_list(tc, &instance->threads) == 1) {
